@@ -30,12 +30,18 @@
 ## give a monotone appearance, but the pattern must be monotone.
 
 'bmonomvn' <-
-function(y, pre=TRUE, p=0.9, B=100, T=200, thin=5,
+function(y, pre=TRUE, p=0.9, B=100, T=200, thin=10,
          method=c("default", "rjlasso", "rjlsr", "lasso"),
-         capm=method!="lasso", r=2, delta=0.1, rao.s2=TRUE,
-         verb=1, trace=FALSE)
+         capm=method!="lasso", start=NULL, r=2, delta=0.1,
+         rao.s2=TRUE, verb=1, trace=FALSE)
   {
-    ## save column namings in a data frame, and then work with a matrix
+    ## (quitely) double-check that blasso is clean before-hand
+    bmonomvn.cleanup(1)
+    
+    ## what to do if fatally interrupted?
+    on.exit(bmonomvn.cleanup(verb))
+    
+    ## save column names in a data frame, and then work with a matrix
     nam <- colnames(y)
     y <- as.matrix(y)
   
@@ -54,8 +60,8 @@ function(y, pre=TRUE, p=0.9, B=100, T=200, thin=5,
       stop("B must be a scalar integer >= 0")
     
     ## check T
-    if(length(T) != 1 || T <= 1)
-      stop("T must be a scalar integer > 1")
+    if(length(T) != 1 || (T <= 0 && B > 0))
+      stop("if B>0 when T must be a scalar integer >= 1")
 
     ## check thin
     if(length(thin) != 1 || thin < 1)
@@ -64,6 +70,12 @@ function(y, pre=TRUE, p=0.9, B=100, T=200, thin=5,
     ## check method
     method <- match.arg(method)
 
+    ## change method into an integer
+    mi <- 0  ## for rjlasso 
+    if(method == "rjlsr") mi <- 1
+    else if(method == "lasso") mi <- 2
+    else if(method == "default") mi <- 3
+    
     ## check capm
     if(length(capm) != 1 || !is.logical(capm))
       stop("capm must be a scalar logical or \"p\"\n")
@@ -71,20 +83,13 @@ function(y, pre=TRUE, p=0.9, B=100, T=200, thin=5,
       warning("capm must be FALSE for method=\"lasso\"", immediate.=TRUE)
       capm <- FALSE
     }
-
-    ## change method into an integer
-    mi <- 0
-    if(method == "rjlsr") mi <- 1
-    else if(method == "lasso") mi <- 2
-    else if(method == "default") mi <- 3
-
+        
     ## save the call
     cl <- match.call()
     
     ## get the number of nas in each column
     nas <- apply(y, 2, function(x) {sum(is.na(x))})
 
-    
     ## check for cols with all NAs
     if(sum(nas == N) > 0) {
       cat("cols with no data:\n")
@@ -99,17 +104,13 @@ function(y, pre=TRUE, p=0.9, B=100, T=200, thin=5,
     } else {
       nao <- 1:ncol(y)
     }
-
+    
+    ## check the start argument
+    start <- check.start(start, nao, M)
+    
     ## number of non-nas in each column
     n <- N - nas[nao]
 
-    ## check for big-p-small-n problems
-    if(any(n >= 0:(length(n)-1))){
-      if(method == "rjlsr" || method == "lasso")
-        warning("methods \"rjlsr\" and \"lasso\" not ideal if any n[i] >= i",
-                immediate.=TRUE)
-    }
-
     ## replace NAs with zeros
     Y <- y
     Y[is.na(Y)] <- 0
@@ -128,6 +129,10 @@ function(y, pre=TRUE, p=0.9, B=100, T=200, thin=5,
             p = as.double(p),
             mi = as.integer(mi),
             capm = as.integer(capm),
+            smu = as.double(start$mu),
+            sS = as.double(start$S),
+            sncomp = as.integer(start$ncomp),
+            slambda = as.double(start$lambda),
             r = as.double(r),
             delta = as.double(delta),
             rao.s2 = as.integer(rao.s2), 
@@ -136,6 +141,7 @@ function(y, pre=TRUE, p=0.9, B=100, T=200, thin=5,
             mu = double(M),
             mu.var = double(M),
             S = double(M*M),
+            S.var = double(M*M),
             methods = integer(M),
             lambda2 = double(M),
             ncomp = double(M),
@@ -146,21 +152,17 @@ function(y, pre=TRUE, p=0.9, B=100, T=200, thin=5,
 
     ## make S into a matrix
     r$S <- matrix(r$S, ncol=M)
+    r$S.var <- matrix(r$S.var, ncol=M)
 
     ## possibly add column permutation info from pre-processing
     if(pre) {
       r$na <- nas
-      r$o <- order(nao)
-      r$pre <- TRUE
-    } else r$pre <- FALSE
-
+      r$o <- nao
+    }
+    
     ## extract the methods
     mnames <- c("bcomplete", "brjlasso", "brjlsr", "blasso", "blsr")
     r$methods <- mnames[r$methods]
-    ##z <- r$lambda2 == 0a
-    ##r$methods[z] <- "blsr"
-    ##r$methods[1] <- "bcomplete"
-    ##r$methods[!z] <- "blasso"
     
     ## put the original ordering back
     if(pre) {
@@ -168,6 +170,7 @@ function(y, pre=TRUE, p=0.9, B=100, T=200, thin=5,
       r$mu <- r$mu[oo]
       r$mu.var <- r$mu.var[oo]
       r$S <- r$S[oo,oo]
+      r$S.var <- r$S.var[oo,oo]
       r$ncomp <- r$ncomp[oo]
       r$lambda2 <- r$lambda2[oo]
       r$methods <- r$methods[oo]
@@ -180,6 +183,7 @@ function(y, pre=TRUE, p=0.9, B=100, T=200, thin=5,
       r$mu.var <- matrix(r$mu.var, nrow=length(r$mu.var))
       rownames(r$mu.var) <- nam
       colnames(r$S) <- rownames(r$S) <- nam
+      colnames(r$S.var) <- rownames(r$S.var) <- nam
       r$ncomp <- matrix(r$ncomp, nrow=length(r$ncomp))
       rownames(r$ncomp) <- nam
       r$lambda2 <- matrix(r$lambda2, nrow=length(r$lambda2))
@@ -187,7 +191,7 @@ function(y, pre=TRUE, p=0.9, B=100, T=200, thin=5,
     }
 
     ## read the trace in the output files, and then delete them
-    if(trace) r$trace <- bmonomvn.read.traces(r$N, r$n, r$m, oo, nam, r$verb)
+    if(trace) r$trace <- bmonomvn.read.traces(r$N, r$n, r$M, oo, nam, r$verb)
     else r$trace <- NULL
     
     ## final line
@@ -195,6 +199,7 @@ function(y, pre=TRUE, p=0.9, B=100, T=200, thin=5,
 
     ## null-out redundancies
     r$n <- r$N <- r$M <- r$mi <- r$verb <- NULL
+    r$smu <- r$sS <- r$sncomp <- r$slambda <- NULL
 
     ## change back to logicals or original inputs
     r$rao.s2 <- as.logical(r$rao.s2)
@@ -207,6 +212,55 @@ function(y, pre=TRUE, p=0.9, B=100, T=200, thin=5,
   }
 
 
+## check.start:
+##
+## sanity check the format of the start vector, and then
+## re-arrange the components into the monotone order
+## specified in nao, which should agree with start$o
+
+check.start <- function(start, nao, M)
+{
+  s <- list(mu=NULL, S=NULL, ncomp=NULL)
+  
+  if(!is.null(start)) {
+    
+    ## make sure orders are the smae
+    if(!is.null(start$o) && start$o != nao)
+      stop("starting monotone order is not the same as y's order")
+    
+    ## check and the reorder mu
+    if(!is.null(start$mu) && length(start$mu) == M) s$mu <- start$mu[nao]
+    else stop("start$mu must be specified and have length ncol(y)")
+    
+    ## check and then reorder S
+    if(!is.null(start$S) && nrow(start$S) == ncol(start$S) && nrow(start$S) == M)
+      s$S <- start$S[nao, nao]
+    else stop("start$S must be specified, be square, and have dim = ncol(y)")
+    
+    ## check and then reorder ncomp
+    if(!is.null(start$ncomp)) {
+      s$ncomp <- start$ncomp
+      na <- is.na(s$ncomp)
+      s$ncomp <- s$ncomp[nao]
+      s$ncomp[na[nao]] <- (0:(length(s$ncomp)-1))[na[nao]]
+      if(length(s$ncomp) != M || !is.integer(s$ncomp) || any(s$ncomp < 0) )
+        stop("start$ncomp should be a non-neg integer vector of length ncol(y)")
+    } else s$ncomp <- 0:(M-1)
+
+    ## check and then reorder lambda
+    if(!is.null(start$lambda)) {
+      s$lambda <- start$lambda
+      na <- is.na(s$lambda)
+      s$lambda <- s$lambda[nao]
+      s$lambda[na[nao]] <- 0
+      if(length(s$lambda) != M || any(s$lambda < 0) )
+        stop("start$lambda should be a non-neg vector of length ncol(y)")
+    } else s$lambda <- rep(0, M)
+
+  }
+  return(s)
+}
+
 
 ## bmonomvn.read.traces:
 ##
@@ -259,8 +313,8 @@ function(y, pre=TRUE, p=0.9, B=100, T=200, thin=5,
 
   ## read the blasso regression traces
   for(i in 1:length(n)) {
-    fname <- paste("blasso_m", i-1, "_n", n[i], ".trace", sep="")
-    lname <- paste("m", i-1, ".n", n[i], sep="")
+    fname <- paste("blasso_M", i-1, "_n", n[i], ".trace", sep="")
+    lname <- paste("M", i-1, ".n", n[i], sep="")
     trace$reg[[lname]] <- read.table(fname, header=TRUE)
     if(rmfiles) unlink(fname)
 
@@ -277,3 +331,38 @@ function(y, pre=TRUE, p=0.9, B=100, T=200, thin=5,
 
   return(trace)
 }
+
+
+## bmonomvn.cleanup
+##
+## gets called when the C-side is aborted by the R-side and enables
+## the R-side to clean up the memory still allocaed to the C-side,
+## as well as whatever files were left open on the C-side
+
+"bmonomvn.cleanup" <-  function(verb)
+{
+  .C("bmonomvn_cleanup", PACKAGE="monomvn")
+
+  ## get rid of trace of the mean samples (mu)
+  if(file.exists(paste("./", "mu.trace", sep=""))) {
+    unlink("mu.trace")
+    if(verb >= 1) cat("NOTICE: removed mu.trace\n")
+  }
+
+  ## get rid of trace of the Covar samples (S)
+  if(file.exists(paste("./", "S.trace", sep=""))) {
+    unlink("S.trace")
+    if(verb >= 1) cat("NOTICE: removed S.trace\n")
+  }
+
+  ## get all of the names of the tree files
+  b.files <- list.files(pattern="blasso_M[0-9]+_n[0-9]+.trace")
+    
+  ## for each tree file
+  if(length(b.files > 0)) {
+    for(i in 1:length(b.files)) {
+      if(verb >= 1) cat(paste("NOTICE: removed ", b.files[i], "\n", sep=""))
+      unlink(b.files[i])
+    }
+  }
+}
